@@ -43,6 +43,10 @@
 #' @param pch Plot symbols for observed data
 #' @param residuals Logical; if \code{TRUE}, plots a residual versus predictor
 #'   plot instead of the usual scale of raw response.
+#' @param main Character title for plot, default is
+#'   \code{"Conditional expectation"}.
+#' @param xlim Graphical parameter passed to plotting functions.
+#' @param ylim Graphical parameter passed to plotting functions.
 #'
 #' @return A list containing relevant information for updating the plot.
 #'
@@ -59,10 +63,13 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
   model.lty = NULL, model.name = NULL, yhat = NULL, mar = NULL, col = "black",
   weights = NULL, view3d = FALSE, theta3d = 45, phi3d = 20, xs.grid
   = NULL, prednew = NULL, conf = FALSE, probs = FALSE, pch = 1, residuals =
-  FALSE)
+  FALSE, main = NULL, xlim = NULL, ylim = NULL)
 {
   ny <- nrow(y)
   col <- rep(col, length.out = ny)
+  main <- if (is.null(main))
+    "Conditional expectation"
+  else main
   dev.hold()
 
 ## If no weights are provided, just show all data with the appropriate colour.
@@ -75,13 +82,8 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
   } else {
     if (!identical(length(weights), ny))
       stop("'weights' should be same length as number of observations")
-    weightsgr0 <- which(weights > 0)
-    data.order <- weightsgr0[order(weights[weightsgr0])]
-    newcol <- (col2rgb(col[data.order]) * matrix(rep(weights[data.order], 3),
-      nrow = 3, byrow = TRUE) / 255) + matrix(rep(1 - weights[data.order], 3),
-      nrow = 3, byrow = TRUE)
-    data.colour <- rep(NA, ny)
-    data.colour[data.order] <- rgb(t(newcol))
+    data.colour <- weightcolor(col, weights)
+    data.order <- attr(data.colour, "order")
   }
 
 ## Organise defaults and check inputs.
@@ -103,10 +105,13 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
   model.lty <- if (is.null(model.lty))
     rep(1, length(model))
   else rep(model.lty, length.out = length(model))
-  model.name <- if(!is.null(names(model)))
+  model.name <- if (!is.null(names(model)))
     names(model)
   else paste("model", seq_along(model), sep = "_")
-  par(mar = c(5, 4, 3, 2))
+  mar <- if (is.null(mar))
+    c(5, 4, 3, 2)
+  else mar
+  par(mar = mar)
 
 ## If xs is NULL, show a univariate summary
 
@@ -136,10 +141,18 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
     if (identical(ncol(xs), 1L)){
       # xs has one column
       if (is.null(xs.grid)){
-        xs.grid <- if (!is.factor(xs[, 1L]))
-          data.frame(seq(min(xs[, 1L], na.rm = TRUE), max(xs[, 1L], na.rm = TRUE
-            ), length.out = if (view3d) {20L} else 50L))
-        else data.frame(as.factor(levels(xs[, 1L])))
+        if (!is.factor(xs[, 1L])){
+          xs.min <- if (is.null(xlim))
+            min(xs[, 1L], na.rm = TRUE)
+          else xlim[1]
+          xs.max <- if (is.null(xlim))
+            max(xs[, 1L], na.rm = TRUE)
+          else xlim[2]
+          xs.grid <- data.frame(seq(xs.min, xs.max, length.out = if (view3d) 20L
+            else 50L))
+        } else {
+          xs.grid <- data.frame(as.factor(levels(xs[, 1L])))
+        }
         colnames(xs.grid) <- colnames(xs)
       }
       newdata <- makenewdata(xs = xs.grid, xc.cond = xc.cond)
@@ -153,7 +166,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
           plot.type <- "ff"
           if (identical(nlevels(y[, 1L]), 2L)){
             plot(unique(xs[, 1L]), rep(-888, length(levels(xs[, 1L]))), col =
-              NULL, main = "Conditional expectation", ylab = paste("Probability
+              NULL, main = main, ylab = paste("Probability
               ", colnames(y)[1L], "=", levels(y[, 1L])[2L]), ylim = c(0, 1))
             if (length(data.order) > 0)
 				      points.default((as.numeric(xs[data.order, 1L])) + rnorm(n =
@@ -177,7 +190,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
             plot(range(as.numeric(xs[, 1L])) + c(0, 0.1 * abs(diff(range(
               as.numeric(xs[, 1L])))) ), range(as.integer(y[, 1L])), col = NULL,
               xlab = colnames(xs)[1L], ylab = colnames(y)[1L], yaxt = "n", main
-              = "Conditional expectation", xaxt = if (is.factor(xs[, 1L])) "n"
+              = main, xaxt = if (is.factor(xs[, 1L])) "n"
               else NULL)
             axis(2, at = 1:nlevels(y[, 1L]), labels = levels(y[, 1L]))
             if (is.factor(xs[, 1L]))
@@ -197,8 +210,8 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
           # y is continuous
           plot.type <- "cf"
           plot(unique(xs[, 1L]), rep(-888, length(levels(xs[, 1L]))), col = NULL
-            , main = "Conditional expectation", xlab = colnames(xs)[1L], ylab =
-            colnames(y)[1L], ylim = range(y[, 1L]))
+            , main = main, xlab = colnames(xs)[1L], ylab =
+            colnames(y)[1L], ylim = if(is.null(ylim)) range(y[, 1L]) else ylim)
           if (length(data.order) > 0)
             points(xs[data.order, 1L], y[data.order, 1L], col = data.colour[
               data.order], pch = pch[data.order])
@@ -232,7 +245,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
           plot.type <- "fc"
           if (identical(nlevels(y[, 1L]), 2L)){
             plot(range(xs[, 1L]) + 0.1 * abs(diff(range(xs[, 1L]))), c(0, 0),
-              col = NULL, main = "Conditional expectation", xlab = colnames(xs)[
+              col = NULL, main = main, xlab = colnames(xs)[
               1L], ylab = paste("Probability ", colnames(y)[1L], "=", levels(y[,
               1L])[2L]), ylim = c(0, 1))
             if (length(data.order) > 0)
@@ -254,7 +267,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
           } else {
             plot(range(xs[, 1L]), range(as.integer(y[, 1L])), col = NULL, xlab =
               colnames(xs)[1L], ylab = colnames(y)[1L], yaxt = "n", main =
-              "Conditional expectation", xaxt = if (is.factor(xs[, 1L])) "n"
+              main, xaxt = if (is.factor(xs[, 1L])) "n"
               else NULL)
             axis(2, at = 1:nlevels(y[, 1L]), labels = levels(y[, 1L]))
             if (is.factor(xs[, 1L]))
@@ -274,8 +287,8 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
           # y is continuous
           plot.type <- "cc"
           plot(range(xs[, 1L]), range(y[, 1L]), col = NULL, main =
-            "Conditional expectation", xlab = colnames(xs)[1L], ylab = colnames(
-            y)[1L], ylim = range(y[, 1L]))
+            main, xlab = colnames(xs)[1L], ylab = colnames(
+            y)[1L], xlim = xlim, ylim = ylim)
           if (length(data.order) > 0){
             points(xs[data.order, 1L], y[data.order, 1L], col = data.colour[
               data.order], pch = pch[data.order])
@@ -346,7 +359,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
 			  plot(xrect, yrect, col = NULL, xlab = colnames(xs)[1L], ylab = colnames(
           xs)[2L], xlim = c(min(xrect) - xoffset, max(xrect) + xoffset), xaxt =
           "n", bty = "n", ylim = c(min(yrect) - yoffset, max(yrect) + yoffset),
-          yaxt = "n", main = "Conditional expectation")
+          yaxt = "n", main = main)
 			  rect(xleft = xrect - xoffset, xright = xrect + xoffset, ybottom = yrect
           - yoffset, ytop = yrect + yoffset, col = color)
         if (length(data.order) > 0)
@@ -374,7 +387,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
 			    yoffset <- abs(diff(unique(yrect)[1:2])) / 2.1
 		      plot(0, 0, col = NULL, xlab = colnames(xs)[!arefactorsxs], ylab =
             colnames(xs)[arefactorsxs], xlim = c(min(xrect) - xoffset, max(xrect
-            ) + xoffset), bty = "n", main = "Conditional expectation", ylim =
+            ) + xoffset), bty = "n", main = main, ylim =
             c(min(yrect) - yoffset, max(yrect) + yoffset), yaxt = "n")
 		      rect(xleft = xrect - xoffset, xright = xrect + xoffset, ybottom =
             yrect - yoffset, ytop = yrect + yoffset, col = color, border = NA)
@@ -392,7 +405,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
             if (probs){
               plot(range(xs.grid[, 1L]), range(xs.grid[, 2L]), col = NULL, xlab
                 = colnames(xs)[1L], ylab = colnames(xs)[2L], main =
-                "Conditional expectation")
+                main)
               pred <- predict1(model[[1L]], newdata = newdata, probability =
                 TRUE, ylevels = levels(y[, 1L]))
               p1 <- extractprobs(model[[1L]], pred)
@@ -411,7 +424,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
               yoffset <- abs(diff(unique(xs.grid[, 2L])[1:2])) / 2
               plot(range(xs.grid[, 1L]), range(xs.grid[, 2L]), col = NULL,
                 xlab = colnames(xs)[1L], ylab = colnames(xs)[2L],
-                main = "Conditional expectation")
+                main = main)
               rect(xleft = xs.grid[, 1L] - xoffset, xright = xs.grid[, 1L]
                 + xoffset, ybottom = xs.grid[, 2L] - yoffset, ytop =
                 xs.grid[, 2L] + yoffset, col = color, border = NA)
@@ -436,7 +449,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
                 = 0.1, z = z, col = colorfacet, zlim = range(y), xlab =
                 colnames(xs)[1L], ylab = colnames(xs)[2L], zlab = colnames(y)[
                 1L], d = 10, ticktype = "detailed", main =
-                "Conditional expectation", theta = theta3d, phi = phi3d))
+                main, theta = theta3d, phi = phi3d))
               if (length(data.order) > 0){
                 points(trans3d(xs[data.order, 1L], xs[data.order, 2L], y[
                   data.order, 1L], pmat = persp.object), col = data.colour[
@@ -453,7 +466,7 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
               yoffset <- abs(diff(unique(xs.grid[, 2L])[1:2])) / 2
               plot(range(xs.grid[, 1L]), range(xs.grid[, 2L]), col = NULL,
                 xlab = colnames(xs)[1L], ylab = colnames(xs)[2L], main =
-                "Conditional expectation")
+                main)
               rect(xleft = xs.grid[, 1L] - xoffset, xright = xs.grid[, 1L] +
                 xoffset, ybottom = xs.grid[, 2L] - yoffset, ytop = xs.grid[,
                 2L] + yoffset, col = color, border = NA)
@@ -467,14 +480,14 @@ function (xs, y, xc.cond, model, model.colour = NULL, model.lwd = NULL,
     }
   }
   dev.flush()
-  structure(list(xs = xs, y = y, xc.cond = xc.cond, model = model, model.colour
-    = model.colour, model.lwd = model.lwd, model.lty = model.lty, model.name =
-    model.name, yhat = yhat, mar = par("mar"), data.colour = data.colour,
-    data.order = data.order, view3d = view3d, theta3d = theta3d, usr = par(
-    "usr"), phi3d = phi3d, plot.type = if (exists("plot.type")) plot.type else
-    NULL, screen = screen(), device = dev.cur(), xs.grid = xs.grid, newdata =
-    newdata, prednew = prednew, xs.grid = xs.grid, conf = conf, probs = probs,
-    pch = pch, col = col, ny = ny), class = "xsplot")
+  invisible(structure(list(xs = xs, y = y, xc.cond = xc.cond, model = model,
+    model.colour = model.colour, model.lwd = model.lwd, model.lty = model.lty,
+    model.name = model.name, yhat = yhat, mar = par("mar"), data.colour =
+    data.colour, data.order = data.order, view3d = view3d, theta3d = theta3d,
+    usr = par("usr"), phi3d = phi3d, plot.type = if (exists("plot.type"))
+    plot.type else NULL, screen = screen(), device = dev.cur(), xs.grid =
+    xs.grid, newdata = newdata, prednew = prednew, xs.grid = xs.grid, conf =
+    conf, probs = probs, pch = pch, col = col, ny = ny), class = "xsplot"))
 }
 
 ## Helper function to produce a legend to go with plotxs
